#pragma once

#include "stdafx.h"

using Microsoft::WRL::ComPtr;

#define XRCHECK(value) XRCheck(#value, __LINE__, value);

inline XrResult XRCheck(const char* what, int line, XrResult xrRes)
{
    if (XR_FAILED(xrRes)) {
        printf("%s failed with code %d at line %d\n", what, xrRes, line);
        abort();
    }
    return xrRes;
}

class IRenderer
{
public:
    IRenderer(XrInstance instance, XrSystemId systemId)
        : m_xrInstance(instance)
        , m_xrSystemId(systemId)
    {
    }
    virtual ~IRenderer() {}

    virtual int64_t getColorTextureFormat() = 0;
    virtual int64_t getDepthMapTextureFormat() = 0;
    virtual void insertColorSwapchain(XrSwapchain handle, uint32_t width, uint32_t height) = 0;
    virtual void insertDepthSwapchain(XrSwapchain handle, uint32_t width, uint32_t height) = 0;
    virtual void update(float deltaTime) = 0;
    virtual void render(XrSwapchain color, XrSwapchain depth, uint32_t colorIndex, uint32_t depthIndex, const XrCompositionLayerProjectionView& view) = 0;
    virtual void* getGraphicsBindings() = 0;

    virtual void copyForWindow(XrSwapchain handle, int index) = 0;
    virtual bool showWindow() = 0;

    XrInstance m_xrInstance;
    XrSystemId m_xrSystemId;
};
