#pragma once

#include <map>
#include <unordered_map>
#include <glm/glm.hpp>
#include <GL/glew.h>
#include <GL/wglew.h>
#include "IRenderer.hpp"

#pragma comment(lib, "opengl32.lib")

class OpenGLRenderer : public IRenderer
{
public:
    OpenGLRenderer(XrInstance instance, XrSystemId systemId, bool mrEnabled);
    ~OpenGLRenderer();

    int64_t getColorTextureFormat() override { return GL_SRGB8_ALPHA8; }
    int64_t getDepthMapTextureFormat() override { return GL_DEPTH24_STENCIL8; }
    void insertColorSwapchain(XrSwapchain handle, uint32_t width, uint32_t height) override;
    void insertDepthSwapchain(XrSwapchain handle, uint32_t width, uint32_t height) override;
    void update(float deltaTime) override;
    void render(XrSwapchain color, XrSwapchain depth, uint32_t colorIndex, uint32_t depthIndex, const XrCompositionLayerProjectionView& view) override;
    void* getGraphicsBindings() override { return &m_graphicsBindings; }

    void copyForWindow(XrSwapchain handle, int index) override { return; }  // todo
    bool showWindow() override { return true; }                             // todo

private:
    bool initRenderTargets(XrSwapchain color, XrSwapchain depth);
    void compileShaders();
    GLuint compileShader(GLenum type, const char* source);
    void createCube(float posX, float posY, float posZ, float alpha);
    void createWindow();

    static LRESULT CALLBACK wndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
    {
        switch (message) {
            case WM_DESTROY: PostQuitMessage(0); break;
            default: return DefWindowProc(hwnd, message, wParam, lParam); break;
        }

        return 0;
    }

    struct RenderTarget;

    RenderTarget getRenderTarget(XrSwapchain color, XrSwapchain depth, uint32_t colorIndex, uint32_t depthIndex);
    static std::vector<XrSwapchainImageOpenGLKHR> getImagesFromSwapChain(XrSwapchain swapchain);

    struct Cube {
        float posX;
        float posY;
        float posZ;
        GLuint vao;
        GLuint vertexBuffer;
        GLuint indexBuffer;
        GLuint indexCount;
    };
    std::vector<Cube> m_cubes;
    float m_cubeRot;

    struct {
        struct {
            float world[16];
            float view[16];
            float projection[16];
        } data;
        GLuint buffer;
    } m_uniformBuffer;

    GLuint m_shaderProgram;

    struct RenderTarget {
        GLuint frameBuffer;
        GLuint depthBuffer;
        GLuint colorBuffer;
    };

    struct Size {
        uint32_t width;
        uint32_t height;
    };

    std::unordered_map<XrSwapchain, Size> m_swapchainSizeMap;
    std::map<std::pair<XrSwapchain, XrSwapchain>, std::vector<std::vector<RenderTarget>>> m_renderTargetsMap;

    HWND m_hwnd = nullptr;
    HDC m_hdc = nullptr;
    HGLRC m_hglrc = nullptr;
    XrGraphicsBindingOpenGLWin32KHR m_graphicsBindings{XR_TYPE_GRAPHICS_BINDING_OPENGL_WIN32_KHR};

    GLuint m_depthBuffer = 0;
};