#include "stdafx.h"
#include <Windows.h>
#include "Window.h"
#include <d3d11.h>
#include <dxgi.h>

#define CHECK_HR(code)                                         \
    {                                                          \
        HRESULT hr = (code);                                   \
        if (FAILED(hr)) {                                      \
            fprintf(stderr, "Error occured %x", (unsigned)hr); \
            abort();                                           \
        }                                                      \
    }

static LRESULT CALLBACK WinProc(HWND handle, UINT msg, WPARAM wparam, LPARAM lparam)
{
    if (msg == WM_DESTROY || msg == WM_CLOSE) {
        PostQuitMessage(0);
        return 0;
    }

    return DefWindowProc(handle, msg, wparam, lparam);
}

Window::Window(ID3D11Device* device, int width, int height)
    : m_width(width)
    , m_height(height)
    , m_device(device)
{
    // Define window style
    WNDCLASS wc = {0};
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WinProc;
    wc.hCursor = LoadCursor(nullptr, IDC_ARROW);
    wc.lpszClassName = "HelloXR";
    RegisterClass(&wc);

    // Create the window
    m_handle = CreateWindow("HelloXR", "HelloXR", WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_VISIBLE, 100, 100, width, height, nullptr, nullptr, nullptr, nullptr);
    createSwapChain();
}

bool Window::runEventLoop() const
{
    DXGI_PRESENT_PARAMETERS parameters{};
    m_swapChain->Present1(0, 0, &parameters);

    MSG msg = {nullptr};
    while (PeekMessage(&msg, nullptr, 0, 0, PM_REMOVE)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);

        if (msg.message == WM_QUIT) {
            return false;
        }
    }

    return true;
}

void Window::createSwapChain()
{
    Microsoft::WRL::ComPtr<IDXGIDevice2> dxgiDevice;
    CHECK_HR(m_device->QueryInterface(__uuidof(IDXGIDevice2), reinterpret_cast<void**>(dxgiDevice.GetAddressOf())));

    Microsoft::WRL::ComPtr<IDXGIAdapter> dxgiAdapter;
    CHECK_HR(dxgiDevice->GetParent(__uuidof(IDXGIAdapter), reinterpret_cast<void**>(dxgiAdapter.GetAddressOf())));

    Microsoft::WRL::ComPtr<IDXGIFactory2> dxgiFactory;
    CHECK_HR(dxgiAdapter->GetParent(__uuidof(IDXGIFactory2), reinterpret_cast<void**>(dxgiFactory.GetAddressOf())));

    DXGI_SWAP_CHAIN_DESC1 swapChainDesc;
    swapChainDesc.BufferCount = 2;
    swapChainDesc.Width = m_width;
    swapChainDesc.Height = m_height;
    swapChainDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
    swapChainDesc.Stereo = false;
    swapChainDesc.SampleDesc.Count = 1;
    swapChainDesc.SampleDesc.Quality = 0;
    swapChainDesc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
    swapChainDesc.Scaling = DXGI_SCALING_NONE;
    swapChainDesc.SwapEffect = DXGI_SWAP_EFFECT_FLIP_DISCARD;
    swapChainDesc.AlphaMode = DXGI_ALPHA_MODE_UNSPECIFIED;
    swapChainDesc.Flags = 0;

    CHECK_HR(dxgiFactory->CreateSwapChainForHwnd(m_device.Get(), m_handle, &swapChainDesc, nullptr, nullptr, m_swapChain.GetAddressOf()));

    ID3D11Texture2D* backBuffer;
    CHECK_HR(m_swapChain->GetBuffer(0, __uuidof(ID3D11Texture2D), reinterpret_cast<void**>(&backBuffer)));
    CHECK_HR(m_device->CreateRenderTargetView(backBuffer, nullptr, m_renderTargetView.GetAddressOf()));
    backBuffer->GetDesc(&m_backBufferDesc);
    backBuffer->Release();
}
